////////////////////////////////////////////////////////////////////////////
// chem2deditor.cpp
// (c) Left 2000 Ricardo Stefani
// This source File is part of chem2D
///////////////////////////////////////////////////////////////////////////

#include "chem2Deditor.h"
#include <qcanvas.h>
#include <qpointarray.h>
#include <qpoint.h>
#include <qinputdialog.h>
#include <qfontdialog.h>
#include <qmessagebox.h>
#include <qpen.h>

#include "chemPenDlg.h"
#include "chemgraphitm.h"

bool operator<=(const QPoint& p1, const QPoint& p2)
{
return( (p1.x() <= p2.x() ) && (p1.y() <= p2.y() ) );
}

bool operator>=(const QPoint& p1, const QPoint& p2)
{
return((p1.x() >= p2.x()) && (p1.y() >= p2.y()));
}


chem2DEditor::~chem2DEditor()
{
 if(cv)
   delete cv;
}

chem2DEditor::chem2DEditor(QCanvas* v, QWidget* parent,
                           const char* name, WFlags f) :
          QCanvasView(v, parent, name, f)
{
  if(!v)
   {
   cv = new QCanvas(1024,768);
   setCanvas(cv);
   }
}

void chem2DEditor::contentsMousePressEvent(QMouseEvent *ev)
{
   
   if(ev->button() == LeftButton) // Left Button Click
   {
        ActualItem = 0;

        if(ev->state() & ShiftButton)
         { // LeftButton Shift
          drawing = false;
          SelectItem(ev);
          return;
         }

        start = ev->pos();
        
        if(!ActualItem)
        {
         drawing = true;
          
         switch(Selected)
           {
            case SELECT_TOOL:
              {
              drawing = false;
              SelectItem(ev);
              break;
              }
             case LINE_TOOL:
               {
                  DrawBond(ev);
                     break;
               }
             case SBUNDER_TOOL:
              {
                   DrawSbunder(ev);
                   break;
              }
             case SBHIGH_TOOL:
              {
                  DrawSbhigh(ev);
                  break;
              }
             case TEXT_TOOL:
              {
                 DrawAtom(ev);
                 break;
              }
            case CPENTANE_TOOL:
              {
                 DrawCpentane(ev);
                 break;
              }
            case CICLE_TOOL:
             {
                DrawCicle(ev);
                break;
             }
            case AROMA_TOOL:
             {
               DrawAromatic(ev);
               break;
             }
            case CPENTADIENE_TOOL:
             {
               DrawCpentadiene(ev);
               break;
             }
           case ROTATE_TOOL:
             {
               drawing = false;
               SelectItem(ev);
               RotateItem(ev);
               break;
             }
        }
      cv->update();
     }
  } else if(ev->button() & RightButton) // elimina Item
       if(ev->state() & ControlButton)
          {
          drawing = false;
          SelectItem(ev);
          delete ActualItem;
          cv->update();
	  } else ProcessPopup(ev);
          
}

void chem2DEditor::contentsMouseMoveEvent(QMouseEvent* ev)
{
   chemGraphItem* itm;
   if ( ActualItem && !drawing) {
         ActualItem->moveBy(ev->pos().x() - start.x(),
		                                  ev->pos().y() - start.y());
	 start = ev->pos();
	 cv->update();
    }

}

void chem2DEditor::contentsMouseReleaseEvent(QMouseEvent* ev)
{
 if(ev->button() == LeftButton)
   if(ActualItem && drawing)
     drawing = false;
 cv->update();
}

// double clicks

void chem2DEditor::contentsMouseDoubleClickEvent(QMouseEvent* ev)
{

}

// popupMenu

void chem2DEditor::ProcessPopup(QMouseEvent* ev)
{
drawing = false;
SelectItem(ev);

if(ActualItem)
{
   if(ActualItem->rtti()==3) // se for Texto
      {
       menuProperties = new QPopupMenu(viewport());
       menuProperties->insertItem( tr("Change Attributes..."), this, SLOT(setFont()) );
      } else
      {
       menuProperties = new QPopupMenu;
       menuProperties->insertItem( tr("Change Attributes..."), this, SLOT(setPen()) );
      }
menuProperties->popup(ev->pos());
}


}

// popupmenu Commands

void chem2DEditor::setPen()
{
 if(!ActualItem)
    return;

 chemPenDlg *dlg = new chemPenDlg(this);
 QCanvasPolygonalItem *it;
 if(dlg->exec())
  {
    it = dynamic_cast<QCanvasPolygonalItem*> (ActualItem);
    if(it)
     it->setPen( dlg->pen );
    cv->update();
  }
}

void chem2DEditor::setFont()
{
bool ok;
if(!ActualItem)
 return;
QCanvasText *tx = dynamic_cast<QCanvasText*> (ActualItem);
if(tx)
  {
   QFont font = QFontDialog::getFont(&ok);
   if(ok)
    tx->setFont( font );
    cv->update();
  }
}

// Draw Tools

void chem2DEditor::SelectItem(QMouseEvent* ev)
{
 QCanvasItem* itTemp; // temporary Item
 ActualItem = 0;

 QCanvasItemList li = cv->allItems(); // Find the Item to Select
   for (QCanvasItemList::Iterator iti = li.begin(); iti!=li.end(); ++iti)
     {
      itTemp = *iti;
      itTemp->setSelected(false);
     } 


 QCanvasItemList l = cv->collisions(ev->pos()); // Find the Item to Select
   for (QCanvasItemList::Iterator it = l.begin(); it!=l.end(); ++it)
     {
      ActualItem = *it;
      ActualItem->setSelected(true);
      start = ev->pos();
      cv->update();
     }

}

void chem2DEditor::RotateItem(QMouseEvent* ev)
{
 chemGraphItem *i = (chemGraphItem *) ActualItem;
 cv->update();

}



void chem2DEditor::DrawBond(QMouseEvent* ev)
{
    chemGraphItem* i = new chemGraphItem(cv);
    i->setType(_GR_SINGLE_BOND);
    i->move(ev->x(), ev->y());
    i->setPen( QPen(black, 1) );
    i->show();
    drawing = true;
    ActualItem = i;
}

void chem2DEditor::DrawAtom(QMouseEvent* ev)
{
  QString symbol = QInputDialog::getText( tr("Atom Symbol"), tr("Enter an atom symbol"),
                                 QString::null);
  QCanvasText *i = new QCanvasText(symbol, cv);
  i->move(ev->x(),ev->y());
  i->show();
  drawing = false;
}

void chem2DEditor::DrawCicle(QMouseEvent* ev)
{
   
    chemGraphItem* i = new chemGraphItem(cv);
    i->setType(_GR_CICLE_HEXANE);
    i->move(ev->x(), ev->y());
    i->setPen( QPen(black, 1) );
    i->show();
    drawing = false;
}

void chem2DEditor::DrawAromatic(QMouseEvent* ev)
{
     chemGraphItem* i = new chemGraphItem(cv);
    i->setType(_GR_AROMATIC);
    i->move(ev->x(), ev->y());
    i->setPen( QPen(black, 1) );
    i->show();
    drawing = false;
}

void chem2DEditor::DrawCpentane(QMouseEvent *ev)
{
    chemGraphItem* i = new chemGraphItem(cv);
    i->setType(_GR_CICLE_PENTANE);
    i->move(ev->x(), ev->y());
    i->setPen( QPen(black, 1) );
    i->show();
    drawing = false;
}


void chem2DEditor::DrawSbunder(QMouseEvent* ev)
{

}

void chem2DEditor::DrawSbhigh(QMouseEvent* ev)
{

}


void chem2DEditor::DrawCpentadiene(QMouseEvent* ev)
{
   chemGraphItem* i = new chemGraphItem(cv);
    i->setType(_GR_CICLE_PENTADIENE);
    i->move(ev->x(), ev->y());
    i->setPen( QPen(black, 1) );
    i->show();
    drawing = false;
}